/********************************************************
 * Data Visualization - 3D                              *
 * Tested in Processing 3                               *     
 *                                                      *
 * Author: Yan Kong                                     *
 * Supervisor: Rodger Luo                               *
 *                                                      *
 * Purpose: Demonstrate the popularity of Quentin       *
 *          Tatantino's movies, correlations between    *
 *          checkouts each year and Quentin's Oscar     *
 *          winning moments. Visualize it via 3D graph. *
 *                                                      *
 * Usage:  Click on each planet, it will only show the  *
 *         clicked planet with a spherical coordinate   *
 *         system. When zooming out, going back original*
 *         system                                       *
 ********************************************************/

import java.util.*; 
import java.util.List; 
import java.lang.Math.*;
import peasy.*;
import java.awt.event.KeyEvent;
import java.util.PriorityQueue;
import java.util.Comparator;
import ComputationalGeometry.*;

IsoWrap[] wrap = new IsoWrap[MovieName.values().length - 1];
PeasyCam cam;
Comparator<Ptpair> comparator = new ComparetorByDist();
// For display window:
final float marginToTop = 40;
final float marginToLeft = 40;

// For data:
final int movies = 9;
final int TEXTSIZE = 15;
final float centerX = 640;
final float centerY = 360;
final int WIDTH = 1280;
final int HEIGHT = 720;
final int YEARS = 12;
final int ITEMTYPE = 0;
final int TITLE = 1;
final int TOTAL_COUNTS = 2;
final int STARTYEAR = 3;
ArrayList<DataPoint[]>[] allPoints = new ArrayList[movies];
ArrayList<Ptpair>[] allLines = new ArrayList[movies];
float[] totalCheckoutsYearly;
Table t1;
Table t2;
boolean drawCoord[] =  new boolean[movies];
ArrayList<Triplet<Itemtype, MovieName, int[]>>[] checkouts = new ArrayList[movies];
float maxCheckouts = -Float.MAX_VALUE, minCheckouts = Float.MAX_VALUE;
float startAngle[] = {random(0, TWO_PI / 10), random(7 *TWO_PI/10, 8 * TWO_PI/10), random(TWO_PI / 10, TWO_PI/5), random(TWO_PI/5, 3*TWO_PI/10), random( 8* TWO_PI/10, TWO_PI), random(3*TWO_PI/10, 2*TWO_PI/5), random(2*TWO_PI/5, TWO_PI/2), 
  random(TWO_PI/2, 3*TWO_PI/5), random( 3*TWO_PI/5, 7 *TWO_PI/10), };
float[] rad = new float[movies];
boolean state = true;
boolean[] clicked = new boolean[movies];
PVector[] currCenter = new PVector[movies];
// For display:
final int radius = 400;

//Only run one time
void setup() {
  // frameRate(10);
  size(1280, 720, P3D);
  //fullScreen();
  // peasyCam setup
  cam = new PeasyCam(this, 1000);
  float  cameraZ = (height / 2.0) / tan(PI * 60.0 / 360.0);
  perspective(PI/3.0, float(width) / float(height), cameraZ / 10.0, cameraZ * 10.0); //set near clip to 0.001 to go inside data
  cam.setWheelScale(0.9);
  cam.setMinimumDistance(100);
  cam.setMaximumDistance(2000);  // clamp zooming
  // process data
  // checkouts.csv format: bibNumber  itemType  title  Counts  2006  2007  2008  2009  2010  2011  2012  2013  2014  2015  2016  2017
  t1 = loadTable("QuentinTarantino_CheckoutCounts.csv", "header");
  for (int i = 0; i < movies; i++)
  {
    checkouts[i] = new ArrayList<Triplet<Itemtype, MovieName, int[]>>();
    allPoints[i] = new ArrayList<DataPoint[]>();
    allLines[i] = new ArrayList<Ptpair>();
    wrap[i] = new IsoWrap(this);
    drawCoord[i] = false;
    clicked[i] = false;
  }

  readData();
  // printData();
  getMaxMin();
  generateDataPoint();
  generateMesh();
  generateWrapping();
  // generateSkeleton();
}

void draw()
{
println("cam.getDistance()", cam.getDistance());
  background(#262626);
  lights(); 
  hint(DISABLE_DEPTH_TEST);

  // drawWrapping();
  drawOrbit();

//println("state = "+state);
  // default mode
  if (state == true)
    drawMovies();
  else {
    for (int i = 0; i < movies; i++)
    {
      if (clicked[i] == true)
      {

        drawSingleMovie(i);

        //clicked[i] = false;
      }
    }

    if (cam.getDistance()==2000) {
      state = true;
      for (int i = 0; i < movies; i++)
        clicked[i] = false;
    }
    
  }



  gui();
}